#!/usr/bin/env python

"""
A 'dead' console config viewer for cisco files.  Attempts to mimic the behavior of looking at a config
file while being logged into the device.  In particular "show run X" to show the stanza for X.

Syntax:  offline-console file1 file2 file3 file4....
"""

import sys
import optparse

import readline
import pydoc #stealing their pager

import subprocess
import tempfile


# generally, we only want to auto-complete on the first word.  Except for these, autocomplete deeper.
deep_parse_rules = {'object-group':2, 'access-list':1, 'interface':1 }

# defaults to 20, you can change it if you want
page_len = 20


class Completer(object):
    def __init__(self,options):
        self.options = sorted(options)
        #print self.options
        return

    def complete(self,text,state):
        response = None
        if state == 0:
            # This is the first time for this text, so build a match list.
            if text:
                self.matches = [s
                                for s in self.options
                                if s and s.startswith(text)]
            else:
                self.matches = self.options[:]
                               
        try:
            response = self.matches[state]
        except IndexError:
            response = None

        return response


def showrun(lines, arg):

    buf = ""
    state = 0

    for line in lines:

        # checking len here because otherwise we get double output on vanilla "show run"
        if state == 1 and len(arg) > 0:
            if line[0] == ' ' or line[0] == '!':
                buf += line
            else:
                state = 0
                
        #@@ fix this to be saner later.  basically, we always want the filename ahead of any stanza that came from that file.
        if line.startswith("\n-=-=-") or line.lower().startswith(arg.lower()):
            buf += line
            state = 1

    return buf


def process_command_line(argv):
    """
    Return a 2-tuple: (settings object, args list).
    `argv` is a list of arguments, or `None` for ``sys.argv[1:]``.
    """
    if argv is None:
        argv = sys.argv[1:]

    # initialize the parser object:
    parser = optparse.OptionParser(
        formatter=optparse.TitledHelpFormatter(width=78),
        add_help_option=None)

    # define options here:
    parser.add_option(      # customized description; put --help last
        '-h', '--help', action='help',
        help='Show this help message and exit.')

    settings, args = parser.parse_args(argv)

    return settings, args

def main(argv=None):

    global page_len
    
    settings, args = process_command_line(argv)
    # application code here, like:
    # run(settings, args)

    lines = []

    for filename in args:
        with open(filename) as f:
            print "Reading " + f.name + "...",
            if len(args) > 1:
                lines.append("\n-=-=-=-=-=-=-=-=-=-" + filename + "-=-=-=-=-=-=-=-=-=-\n")
            lines.extend(f.readlines())
            print "done"
                    
    biglist = []
            
    for line in lines:
        
        words = line.strip().split()

        if len(words) == 0:
            continue

        if words[0] not in biglist:
            # always interested in the first word of the line
            biglist.append(words[0])

        # sometimes interested in deeper words
        if (words[0] in deep_parse_rules):

            for i in range(0,deep_parse_rules[words[0]]):
                if words[i+1] not in biglist:
                    biglist.append(words[i+1])

    print str(len(biglist)) + " words added to autocomplete"
    
    readline.set_completer(Completer(biglist).complete)
    readline.set_completer_delims(' ')    
    readline.parse_and_bind('tab: complete')


    print "-----------------------------------------------------------"
    print "WARNING: The PIPE (|) is ALIVE.  It runs LIVE IN YOUR SHELL"
    print "It is NOT SIMULATED within this console.  DO NOT do stupid."
    print "-----------------------------------------------------------"
    print
    print "Commands: ('show run ARG', 'pager NUMLINES', 'exit')"
    print

    l = ''
    while l != 'exit':
        l = raw_input('>')
        
        args = l.split()

        if len(args) > 0:

            if args[0] == "autocomp":
                print(biglist)

            if args[0] == "pager":
                if len(args) > 1 and int(args[1]) > 0:
                    page_len = int(args[1])
                    print "pager set: %s" % args[1]
                else:
                    print "pager is %s lines" % page_len

            
            if args[0] == "sh" or args[0] == "sho" or args[0] == "show":
                if len(args) > 1 and (args[1] == "r" or args[1] == "ru" or args[1] == "run"):
                    target = ' '.join(args[2:])
                else:
                    target = ' '.join(args[1:])
                
                if '|' in target:

                    target_splitpipe = target.split('|')
                    target = target_splitpipe[0]
                    target_rest = ' '.join(target_splitpipe[1:])

                    output = showrun(lines, target)

                    with tempfile.NamedTemporaryFile() as temp:
                        temp.write(output)
                        temp.flush()

                        cmd = "cat " + temp.name + " | " + target_rest

                        # subprocess doesn't like it if your pipeline returns non-zero
                        try:
                            output = subprocess.check_output( cmd, shell=True)
                        except subprocess.CalledProcessError as e:
                            output = e.output
                        
                else:
                    output = showrun(lines,target)

                # we want it paged for convenience, but also to linger
                # this makes a spammy mess real fast before the pager kicks in, but whatever.
                print output

                if len(output.split('\n')) > page_len and page_len > 0:                
                    pydoc.pager(output)
    
    return 0        # success

if __name__ == '__main__':
    status = main()
    sys.exit(status)
